from builtins import Exception

from flask import Flask, render_template, request, session
from business.build_json.generate_json import convert_to_json
from business.validate_model.validate_model import validate
from business.authentication.auth import authenticate_user
from business.parse_xml.parse_data import parse

from business.reportings.reportings_manager import save, get_all, get_all_alert_type, \
    get_all_danger_level, filter_alerts, save_html_representation, get_alert_html, get_alert_map, save_comment, get_alert_comments
from business.reportings.cities_manager import get_cities


'''

Controlador principal del prototipo
'''
app = Flask(__name__)


@app.route('/', methods=['GET', 'POST'])
def login():
    if request.method == "POST":
        username = request.form['username']
        password = request.form['password']
        user_auth = authenticate_user(username, password)
        if user_auth:
            session['username'] = username
            incidences = get_all()
            cities = get_cities()
            alert_types = get_all_alert_type()
            danger_levels = get_all_danger_level()
            return render_template('main.html', data=None, username=username, incidences=incidences, cities=cities, \
                    alert_types=alert_types, danger_levels=danger_levels)
        else:
            return render_template('login.html', error='Invalid username or password')
    else:
        return render_template('login.html')


@app.route('/main', methods=['GET', 'POST'])
def main():
    username = session['username']
    incidences = get_all()
    cities = get_cities()
    alert_types = get_all_alert_type()
    danger_levels = get_all_danger_level()
    return render_template('main.html', data=None, username=username, incidences=incidences, cities=cities, \
            alert_types=alert_types, danger_levels=danger_levels)


@app.route('/editor', methods=['GET', 'POST'])
def editor():
    username = session['username']
    if request.method == "POST":
        data = request.form['data']
        action = request.form['action']
        try:
            xmi, validation, valid, html = validate(data, username)
            json = convert_to_json(xmi)
            if action == 'save':
                _id = save(json)
                save_html_representation(_id, html)
        except Exception as validation:
            return render_template('editor.html', data=parse(data), validation=[validation], \
                                   valid=False, username=username)
        return render_template('editor.html', data=parse(data), validation=validation, \
                               valid=valid, json=json, username=username)
    else:
        return render_template('editor.html', data=None, valid=False, username=username)


@app.route('/filter', methods=['POST'])
def filter():
    city = request.form['cities']
    alert = request.form['alerts']
    damage = request.form['damages']
    username = session['username']
    incidences = get_all()
    cities = get_cities()
    alert_types = get_all_alert_type()
    danger_levels = get_all_danger_level()
    filtered_alerts = filter_alerts(city, alert, damage)
    return render_template('main.html', data=None, incidences=filtered_alerts, cities=cities, \
        alert_types=alert_types, danger_levels=danger_levels, city=city, alert=alert, damage=damage, username=username)


@app.route('/alert/<id>', methods=['GET'])
def alert(id):
    alert = get_alert_html(id)
    username = session['username']
    map = get_alert_map(id)
    comments = get_alert_comments(id)
    return render_template('alert.html', alert=alert, map=map, username=username, id=id, comments=comments)


@app.route('/add_comment', methods=['POST'])
def add_comment():
    comment = request.form['comment']
    id = request.form['alert']
    username = request.form['username']
    save_comment(id, username, comment)
    alert = get_alert_html(id)
    map = get_alert_map(id)
    comments = get_alert_comments(id)
    return render_template('alert.html', alert=alert, map=map, username=username, id=id, comments=comments)


if __name__ == '__main__':
    app.secret_key = 'sh58YuiOjhgvNB32FG'
    app.config['SESSION_TYPE'] = 'filesystem'
    app.run(host='127.0.0.1')
