from visitor import Visitor
from business.parse_xml.parse_data import convert_string as c
import datetime

'''

Genera un archivo en formato HTML aplicando el patrón Visitor para la visualización de las incidencias
'''



class HTMLVisitor(Visitor):

    def __init__(self):
        self.doc = None

    def visit_User(self, node):
        self.doc = '<ul class="tree">'
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul>'])
        return self.doc

    def visit_City(self, node):
        self.doc = '\n'.join([self.doc, '<li><div class="icon selected"><img src="/static/images/diagram/city.png" height="32px" width="32px"/><span> City</span></div><ul><li>{} ({})</li>'\
            .format(c(node.name), c(node.country))])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Map(self, node):
        for child in node.children:
            self.visit(child)

    def visit_Alert(self, node):
        self.doc = '\n'.join([self.doc, '<li><div class="icon selected"><img src="/static/images/diagram/{}.png" height="32px" width="32px"/><span> {}</span></div><ul><li>[{}] - {}<li>Danger: {}</li>'\
            .format(node.alert_type, c(node.alert_type), datetime.datetime.now().strftime("%d-%m-%Y %H:%M:%S"), node.description, c(node.danger_level.split('-')[1].replace('_', ' ')))])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Object(self, node):
        self.doc = '\n'.join([self.doc, '<li><div class="icon selected"><img src="/static/images/diagram/{}.png" height="32px" width="32px"/><span> {}</span></div><ul><li>{} || {}</li>'\
                             .format(node.object_type, c(node.object_type), c(node.name), node.description)])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Service(self, node):
        self.doc = '\n'.join([self.doc, '<li><div class="icon selected"><img src="/static/images/diagram/{0}.png" height="32px" width="32px"/><span> Service</span></div><ul><li>{0}</li>'\
                             .format(node.service_type)])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Tweet(self, node):
        self.doc = '\n'.join([self.doc,
                              '<li><div class="icon selected"><img src="/static/images/diagram/tweet.png" height="32px" width="32px"/><span> Tweet</span></div><ul><li>Text: {}</li>' \
                             .format(node.text)])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Geolocation(self, node):
        self.doc = '\n'.join([self.doc,
                              '<li><div class="icon selected"><img src="/static/images/diagram/geolocation.png" height="32px" width="32px"/><span> Geolocation</span></div><ul><li>Position: {}</li>' \
                             .format(node.coordinates)])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Video(self, node):
        self.doc = '\n'.join([self.doc,
                              '<li><div class="icon selected"><img src="/static/images/diagram/video.png" height="32px" width="32px"/><span> Video</span><div class="media"><iframe width="420" height="315" src="{}"></iframe></div></div><ul><li></li>' \
                             .format(node.url)])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])

    def visit_Image(self, node):
        self.doc = '\n'.join([self.doc,
                              '<li><div class="icon selected"><img src="/static/images/diagram/image.png" height="32px" width="32px"/><span> Image</span><div class="media"><img src="{}" alt="Image"/></div></div><ul><li></li>' \
                             .format(node.url)])
        for child in node.children:
            self.visit(child)
        self.doc = '\n'.join([self.doc, '</ul></li>'])