import time

import xml.etree.ElementTree as et

from model.user import User
from model.city import City
from model.map import Map
from model.alert import Alert
from model.object import Object
from model.tweet import Tweet
from model.geolocation import Geolocation
from model.image import Image
from model.video import Video
from model.service import Service
from model.sensor import Sensor

from model.edge import Edge

from business.parse_xml.check_elements import check_main_elements
'''

Convierte los nodos del archivo XML en nodos objeto
'''

def read_xml(xml, username):
    nodes = []
    edges = []
    tree = et.fromstring(xml)
    root = tree.find('root')
    for child in tree.find('root').iter():
        if child.tag == 'Shape':
            nodes.append(create_node(child, username))
        if child.tag == 'Connector':
            edges.append(create_edge(child))
    check_main_elements(nodes)
    return nodes, edges


def create_node(node, username):
    if node.attrib['type'] == 'user':
        return User(node.attrib['id'], username)
    if node.attrib['type'] == 'city':
        return City(node.attrib['id'], node.attrib['_name'], node.attrib['_country'])
    if node.attrib['type'] == 'map':
        return Map(node.attrib['id'], node.attrib['_mapType'].split('-')[1].lower(), node.attrib['_zoom'], \
                   node.attrib['_radius'], node.attrib['_coordinates'])
    if node.attrib['type'] == 'alert':
        return Alert(node.attrib['id'], str(time.time()).split('.')[0], \
                     node.attrib['subtype'].upper(), node.attrib['_description'], node.attrib['_danger_level'])
    if node.attrib['type'] == 'service':
        return Service(node.attrib['id'], node.attrib['subtype'])
    if node.attrib['type'] == 'object':
        return Object(node.attrib['id'], node.attrib['_name'], node.attrib['_description'], \
                           node.attrib['_danger_level'].upper().split('-')[1], node.attrib['subtype'].upper())
    if node.attrib['subtype'] == 'tweet':
        return Tweet(node.attrib['id'], node.attrib['_text'])
    if node.attrib['subtype'] == 'geolocation':
        return Geolocation(node.attrib['id'], node.attrib['_coordinates'])
    if node.attrib['subtype'] == 'img':
        return Image(node.attrib['id'], node.attrib['_url'])
    if node.attrib['subtype'] == 'video':
        return Video(node.attrib['id'], node.attrib['_url'])
    if node.attrib['subtype'] == 'sensor':
        return Sensor(node.attrib['id'], node.attrib['_sensorType'], node.attrib['_value'])
    return None


def create_edge(edge):
    return Edge(edge.find('mxCell').attrib['source'], edge.find('mxCell').attrib['target'])

