import os
from lxml import etree
import configparser

from business.build_xmi.generate_xmi import generate
from business.build_html.generate_html import generate_html

dir_path = os.path.dirname(os.path.realpath(__file__))
config = configparser.ConfigParser()
config.read(dir_path + '/config.ini')


def validate_xmi(xmi):
    xmi = format_xmi(xmi)
    xmlschema_doc = etree.parse(dir_path + '/schema.xsd')
    xmlschema = etree.XMLSchema(xmlschema_doc)
    doc = etree.fromstring(xmi.encode('ascii'))
    try:
        xmlschema.assertValid(doc)
        return ['Valid reporting']
    except etree.DocumentInvalid as err:
        return err.error_log
    except:
        return ['Unknown error, exiting.']


def format_xmi(xmi):
    xmi = config.get('DEFAULT', 'xml_header') + config.get('DEFAULT', 'xmi_header') \
          + xmi + config.get('DEFAULT', 'xmi_ending')
    return xmi


def validate(data, username):
    xmi = generate(data, username)
    validation = validate_xmi(xmi)
    html_representation = generate_html(data, username)
    valid = True if validation[0] == 'Valid reporting' else False
    return xmi, validation, valid, html_representation
