import json
import datetime
import objectpath
from bson.objectid import ObjectId

from model.alert import Alert


class Incidence:

    def __init__(self, database):
        self.database = database

    def save_incidence(self, incidence):
        col = self.database["incidences"]
        return col.insert(json.loads(incidence))

    def get_html_representation(self, id):
        col = self.database["visualizations"]
        cursor = col.find({"id": ObjectId(id)})
        for document in cursor:
            return self.find_value_dict(document, '$..html')

    def save_html_incidence(self, _id, html):
        col = self.database["visualizations"]
        col.insert({'id': _id, 'html': html})

    def save_comment(self, alert, user, comment):
        col = self.database["comments"]
        col.insert({'alert': alert, 'user': user, 'comment': comment,
                    'time': datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S")})

    def get_alert_map(self, id):
        alerts = []
        col = self.database["incidences"]
        cursor = col.find({"_id": ObjectId(id)})
        for document in cursor:
            return self.find_value_dict(document, '$..map')

    def get_alert_comments(self, id):
        comments = []
        col = self.database["comments"]
        cursor = col.find({ "alert": id })
        for document in cursor:
            comments.append(document)
        return comments

    def get_all_incidences(self):
        alerts = []
        col = self.database["incidences"]
        cursor = col.find({})
        for document in cursor:
            id = str(self.find_value_dict(document, '$.._id'))
            city = self.find_value_dict(document, '$..@name')
            alert_type = self.find_value_dict(document, '$..@alert_type')
            description = self.find_value_dict(document, '$..@description')
            danger_level = self.find_value_dict(document, '$..@danger_level')
            alerts.append(Alert(id, city, alert_type, description, danger_level))
        return alerts

    def get_all_alert_types(self):
        return self.get_attributes('$..@alert_type')

    def get_all_danger_level(self):
        return self.get_attributes('$..@danger_level')

    def filter_alerts_by_criteria(self, city, alert, damage):
        alerts = []
        col = self.database["incidences"]
        cursor = col.find({})
        for document in cursor:
            id = str(self.find_value_dict(document, '$.._id'))
            danger_level = self.find_value_dict(document, '$..@danger_level')
            alert_type = self.find_value_dict(document, '$..@alert_type')
            city_name = self.find_value_dict(document, '$..@name')
            description = self.find_value_dict(document, '$..@description')
            alerts.append(Alert(id, city_name, alert_type, description, danger_level))
        alerts = [c for c in alerts if c.city == city] if city != 'all' else alerts
        alerts = [a for a in alerts if a.alert_type == alert] if alert != 'all' else alerts
        alerts = [a for a in alerts if a.danger_level == damage] if damage != 'all' else alerts
        return alerts

    def find_value_dict(self, dictionary, key):
        json_tree = objectpath.Tree(dictionary)
        return tuple(json_tree.execute(key))[0]

    def get_attributes(self, value):
        values = []
        col = self.database["incidences"]
        cursor = col.find({})
        for document in cursor:
            concept = self.find_value_dict(document, value)
            if concept not in values:
                values.append(concept)
        return values
