function onInit(editor) {
    mxVertexHandler.prototype.rotationEnabled = true;
    mxGraphHandler.prototype.guidesEnabled = true;
    mxGuide.prototype.isEnabledForEvent = function (evt) {
        return !mxEvent.isAltDown(evt);
    };
    mxEdgeHandler.prototype.snapToTerminals = true;
    mxConnectionHandler.prototype.connectImage = new mxImage('static/images/arrow.png', 16, 16);
    mxConnectionHandlerInsertEdge = mxConnectionHandler.prototype.insertEdge;
    mxConnectionHandler.prototype.insertEdge = function (parent, id, value, source, target, style) {
        if (target.getEdgeCount() <= target.getChildCount()) {
            return mxConnectionHandlerInsertEdge.apply(this, arguments);
        }
    };
    editor.graph.setConnectable(true);
    editor.graph.setAllowDanglingEdges(false);
    editor.graph.setTooltips(false);
    editor.graph.setCellsResizable(false);

    var node = document.getElementById('mainActions');
    var buttons = ['cut', 'copy', 'paste', 'delete', 'undo', 'redo'];
    var icons = ['fa-scissors', 'fa-files-o', 'fa-clipboard', 'fa-eraser', 'fa-undo', 'fa-repeat'];
    for (var i = 0; i < buttons.length; i++)
    {
        var button = document.createElement('button');
        button.classList.add('btn-two');
        button.classList.add('mini');
        var icon = document.createElement("i");
        icon.innerHTML = "<i class='fa " + icons[i] + "'></i>";
        button.appendChild(icon);
        mxUtils.write(button, ' ' + buttons[i].charAt(0).toUpperCase() + buttons[i].substr(1, buttons[i].length-1));
    
        var factory = function(name)
        {
            return function()
            {
                editor.execute(name);
            };
        };
    
        mxEvent.addListener(button, 'click', factory(buttons[i]));
        node.appendChild(button);
    }

    var title = document.getElementById('title');
    if (title != null) {
        var f = function (sender) {
            title.innerHTML = sender.getTitle();
        };
        editor.addListener(mxEvent.ROOT, f);
        f(editor);
    }
    mxEvent.addMouseWheelListener(function (evt, up) {
        if (!mxEvent.isConsumed(evt)) {
            if (up) {
                editor.execute('zoomIn');
            }
            else {
                editor.execute('zoomOut');
            }
            mxEvent.consume(evt);
        }
    });
    editor.graph.addListener(mxEvent.CLICK, function (sender, evt) {
        var cell = evt.getProperty("cell");
        if (cell != null) {
            try {
                printAttributes(editor, cell.id, cell.value.attributes);
                evt.consume();
            }catch(error){
                document.getElementById('properties').innerHTML = "";
            }
        }
        //document.getElementById("save").style.visibility = "hidden";
    });

    document.getElementById("validate").onclick = function () {
        var enc = new mxCodec();
        var node = enc.encode(editor.graph.getModel());
        document.getElementById('action').value = 'validate';
        document.getElementById('data').value = mxUtils.getPrettyXml(node);
        document.getElementById("submit").submit();
    };

    if (document.getElementById("save") != null)
    {
        document.getElementById("save").onclick = function () {
            var enc = new mxCodec();
            var node = enc.encode(editor.graph.getModel());
            document.getElementById('action').value = 'save';
            document.getElementById('data').value = mxUtils.getPrettyXml(node);
            document.getElementById("submit").submit();
        };
    }

    node = document.getElementById('zoomActions');
    buttons = ['in', 'out', 'actual', 'fit'];
    icons = ['fa-plus-square', 'fa-minus-square', 'fa-square', 'fa-square-o'];
    actions = ['zoomIn', 'zoomOut', 'actualSize', 'fit'];
    for (var i = 0; i < buttons.length; i++) {
        var button = document.createElement('button');
        button.classList.add('btn-two');
        button.classList.add('mini');
        var icon = document.createElement("i");
        icon.innerHTML = "<i class='fa " + icons[i] + "'></i>";
        button.appendChild(icon);
        mxUtils.write(button, ' ' + buttons[i].charAt(0).toUpperCase() + buttons[i].substr(1, buttons[i].length - 1));

        var factory = function (name) {
            return function () {
                editor.execute(name);
            };
        };

        mxEvent.addListener(button, 'click', factory(actions[i]));
        node.appendChild(button);
    }
    var margin = 2;
    var max = 3;

    var bounds = editor.graph.getGraphBounds();
    var cw = editor.graph.container.clientWidth - margin;
    var ch = editor.graph.container.clientHeight - margin;
    var w = bounds.width / editor.graph.view.scale;
    var h = bounds.height / editor.graph.view.scale;
    var s = Math.min(max, Math.min(cw / w, ch / h));

    editor.graph.view.scaleAndTranslate(s,
      (margin + cw - w * s) / (2 * s) - bounds.x / editor.graph.view.scale,
      (margin + ch - h * s) / (2 * s) - bounds.y / editor.graph.view.scale);
    for(i=0; i<=5; i++)
        editor.graph.zoomOut();
}
window.onbeforeunload = function () { return mxResources.get('changesLost'); };

function start(data){
    editor = createEditor('static/config/diagrameditor.xml');
    if(data != 'None'){
        var doc = mxUtils.parseXml(data);
        var codec = new mxCodec(doc);
        codec.decode(doc.documentElement, editor.graph.getModel());
        editor.graph.fit();
        for(i=0; i<5; i++)
            editor.graph.zoomOut();
    }
    var codeout = CodeMirror.fromTextArea(document.getElementById('output'), {
        mode: "javascript",
        theme: "default",
        lineNumbers: true,
        readOnly: true
    });
}

function loadItems(item){
    var tool = document.getElementById('toolbar');
    var images = tool.getElementsByTagName("img");
    var i, elements = document.getElementsByClassName('dright');
    for (i = elements.length; i--;) {         
        elements[i].parentNode.removeChild(elements[i]);             
    }
    for(i=0; i<images.length; i++){
        images[i].className = "imgvisible";
        if(images[i].src.replace(/^.*[\\\/]/, '').startsWith(item)){
            var div = document.createElement("div");
            images[i].classList.add("dleft");
            images[i].classList.add("mxToolbarModeSelected");
            images[i].classList.add("mxToolbarMode");
            div.innerHTML = images[i].title;
            div.className= "dright";
            tool.insertBefore(div, images[i]);
        }else{
            images[i].className = "imghidden";
        }
    }
}

function update(editor, name){
    var attr = name.split('_')[1];
    var value = document.getElementById(name).value;
    editor.graph.getSelectionCell().value.attributes['_' + attr].value = value;
}

function updateselect(editor, name, selected){
    var attr = name.substring(1, name.length);
    var value = document.getElementById(name).value;
    var original = editor.graph.getSelectionCell().value.attributes[attr].value;
    editor.graph.getSelectionCell().value.attributes[attr].value = original.split('-')[0] + '-' + selected;
}

function findValue(id, attr, editor){
    return editor.graph.getSelectionCell().value.attributes[attr].value;
}

function getLocation() {
  if (navigator.geolocation) {
    return navigator.geolocation.getCurrentPosition(showPosition);
  }
}

function showPosition(position) {
    var lat = position.coords.latitude;
    var long = position.coords.longitude;
    return [lat, long];
}

function toUpper(string)
{
    return string.charAt(0).toUpperCase() + string.slice(1).replace('_', ' ');
}

function printAttributes(editor, id, attributes){
    document.getElementById('properties').innerHTML = "";
    for(i=0; i<attributes.length; i++){
        if(attributes[i].name.startsWith('_')){
            var attr = attributes[i].name;
            var value = attributes[i].value;
            var node = document.createElement('div');
            if(! value.includes(';'))
            {      
                node.innerHTML = '<label for="' + String(id)+attr + '">'+ toUpper(attr.substring(1, attr.length)) +': </label>'
                    + '</br><input type="text" id="' + String(id) + attr + '" name="' + String(id)+attr + '"/>';
                document.getElementById('properties').appendChild(node);
                document.getElementById(String(id) + attr).value = findValue(id, attr, editor);
                if(attr == '_coordinates' && findValue(id, attr, editor) == '') {
                    navigator.geolocation.getCurrentPosition(function(position){
                        document.getElementById(String(id) + attr).value = position.coords.latitude + ',' + position.coords.longitude;
                        update(editor, String(id) + attr);
                    });
                }
                document.getElementById(String(id)+attr).onkeyup = function(){
                    update(editor, this.name);
                };
            }
            else {
                var selected = value.split('-')[1];
                value = value.split('-')[0];
                var label = '<label for="' + String(id) + attr + '">' + toUpper(attr.substring(1, attr.length)) + ': </label>';
                var select = '<select id="' + String(id) + attr + '" name="' + String(id) + attr + '">';
                for (j = 0; j < value.split(';').length; j++) {
                    select += '<option value="' + value.split(';')[j] + '">' + value.split(';')[j] + '</option>';
                }
                select += "</select>";
                node.innerHTML = label + '</br>' + select;
                document.getElementById('properties').appendChild(node);
                document.getElementById(String(id) + attr).value = selected;
                document.getElementById(String(id)+attr).onchange = function(){
                    updateselect(editor, this.name, this.value);
                };
            }
        }
    }
}